/*
 * Direct3D FPS counter
 *
 * Copyright 2003 TransGaming Technologies, Inc.
 */

#include "config.h"
#include "wine/port.h"

#include <unistd.h>
#include <signal.h>
#include <stdio.h>

#include "x11drv.h"

#include "bitmap.h"

#include "wine/debug.h"

#ifdef HAVE_OPENGL
#include "wine_gl.h"

#include "glx_private.h"
#endif

DEFAULT_DEBUG_CHANNEL(x11drv);

/* hm... perhaps font design is not my forte. */

static const char numbers[32][80] = {
/* 0      ..       1      ..       2      ..       3      ..       4      ..       */
  "     XXXXX             X            XXXXXXX         XXXXXXX                 X   ",
  "   XX     XX          XX          XX       XX     XX       XX              XX   ",
  "  X         X        X X         X           X   X           X            X X   ",
  " X           X      X  X        X             X               X          X  X   ",
  "X             X    X   X                      X               X         X   X   ",
  "X             X        X                      X              X         X    X   ",
  "X             X        X                     X             XX         X     X   ",
  "X             X        X                   XX        XXXXXX          X      X   ",
  "X             X        X                 XX                XX       X       X   ",
  "X             X        X               XX                    X     X        X   ",
  "X             X        X             XX                       X   X         X   ",
  " X           X         X           XX                         X  XXXXXXXXXXXXXX ",
  "  X         X          X         XX              X           X              X   ",
  "   XX     XX           X        X                 XX       XX               X   ",
  "     XXXXX         XXXXXXXXX    XXXXXXXXXXXXXXX     XXXXXXX                 X   ",
  "                                                                                ",
/* 5      ..       6      ..       7      ..       8      ..       9      ..       */
  "XXXXXXXXXXXXXXX      XXXXXX     XXXXXXXXXXXXXXX      XXXXX          XXXXXXX     ",
  "X                  XX      XX   X            X     XX     XX      XX       XX   ",
  "X                 X          X              X     X         X    X           X  ",
  "X                X                         X     X           X  X             X ",
  "X               X                         X      X           X  X             X ",
  "XXXXXXXXXXX     X   XXXXXXX              X        X         X   X             X ",
  "           XX   X XX       XX           X          XXXXXXXXX    X             X ",
  "             X  XX           X          X         X         X    X           XX ",
  "              X X             X        X         X           X    XX       XX X ",
  "              X X             X        X        X             X     XXXXXXX   X ",
  "              X X             X        X        X             X               X ",
  "X             X X             X        X        X             X              X  ",
  " X           X   X           X         X         X           X   X          X   ",
  "  XX       XX     XX       XX          X          XX       XX     XX      XX    ",
  "    XXXXXXX         XXXXXXX            X            XXXXXXX         XXXXXX      ",
  "                                                                                "
};

#define COUNTS 5
#define INTERVAL 200

static DWORD last_fps, last_time, cur_count, cur_time;
static DWORD count_ptr, count_list[COUNTS], time_list[COUNTS];

extern int show_fps;

static void compute_fps(void)
{
  DWORD ctime = GetTickCount(), delta;
  count_list[count_ptr]++;
  delta = ctime - last_time;
  if (delta > INTERVAL) {
    /* time to update fps */
    time_list[count_ptr] = delta;
    cur_count += count_list[count_ptr];
    cur_time += time_list[count_ptr];
    last_fps = cur_count * 1000 / cur_time;
    last_time = ctime;
    if (++count_ptr >= COUNTS) count_ptr = 0;
    cur_count -= count_list[count_ptr];
    cur_time -= time_list[count_ptr];
    count_list[count_ptr] = 0;
    time_list[count_ptr] = 0;
  }
}

#ifdef HAVE_OPENGL
void X11DRV_GLX_DisplayFPS(LPD3DCONTEXT ctx, LPD3DTARGET tgt)
{
  char text[16];
  unsigned u, x, y, r, c, pass, ofs;

  if (!show_fps) return;

  compute_fps();
  sprintf(text, "%3ld", last_fps);
  X11DRV_GLX_StartDraw(ctx, tgt, GL_COLOR_BUFFER_BIT | GL_TEXTURE_BIT, GL_CLIENT_VERTEX_ARRAY_BIT);

  glEnable(GL_TEXTURE_2D);
  if (ctx->hud_font) glBindTexture(GL_TEXTURE_2D, ctx->hud_font);
  else {
    BYTE *conv;

    conv = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, 128*32);
    for (y=0; y<32; y++) {
      for (x=0; x<80; x++) {
        conv[y*128+x] = (numbers[y][x] == 'X') ? 255 : 0;
      }
    }

    glGenTextures(1, &ctx->hud_font);
    glBindTexture(GL_TEXTURE_2D, ctx->hud_font);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, 128, 32, 0, GL_ALPHA, GL_UNSIGNED_BYTE, conv);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
    HeapFree(GetProcessHeap(), 0, conv);
  }
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  y = 0; ofs = 0;
  glBegin(GL_QUADS);
  for (pass=0; pass<2; pass++) {
    switch (pass) {
    case 0:
      glColor4f(0.0, 0.0, 0.0, 0.75);
      ofs = 1;
      break;
    case 1:
      glColor4f(0.5, 1.0, 0.5, 1.0);
      ofs = 0;
      break;
    }
    for (u=0, x=0; text[u]; u++, x+=16) {
      if (text[u] == ' ') continue;
      c = text[u] - '0';
      r = c/5;
      c -= r*5;
      glTexCoord2f(c/8.0,     r/2.0);     glVertex2f(x+ofs   , y+ofs   );
      glTexCoord2f((c+1)/8.0, r/2.0);     glVertex2f(x+ofs+16, y+ofs   );
      glTexCoord2f((c+1)/8.0, (r+1)/2.0); glVertex2f(x+ofs+16, y+ofs+16);
      glTexCoord2f(c/8.0,     (r+1)/2.0); glVertex2f(x+ofs   , y+ofs+16);
    }
  }
  glEnd();

  X11DRV_GLX_EndDraw(ctx, GL_TEXTURE_BIT, GL_CLIENT_VERTEX_ARRAY_BIT);
}
#endif
